OnePixelLineDown
;requires HL as screen address
	INC H ;go to new line
	LD A,H
	AND 00000111B
	OR A ;h=0 -> going to new character line
	RET NZ; new line belongs to the same character line
	
	LD A,L ;going to new character line
	ADD A,32
	LD L,A
	RET C;going to new screen segment, if e.g h was 71, now it is 72
	
	LD A,H
	SUB 8
	LD H,A; decrease h if we are in the same screen segment
	RET
;---------
OnePixelLineUp
;requires HL as screen address
	DEC H ;go to new line
	LD A,H
	AND 00000111B
	CP 7 ;h=111b -> going to new character line
	RET NZ; new line belongs to the same character line
	
	LD A,L ;going to new character line
	SUB 32
	LD L,A
	RET C;going to new screen segment, if e.g h was 71, now it is 72
	
	LD A,H
	ADD A,8
	LD H,A; decrease h if we are in the same screen segment
	RET
;-------------
XYtoHL
;turn XY into HL screen coordinates
;B contains X, C contains Y 
;doesn't destroy BC
 
	LD  A,C
	AND  00000111b
	LD  H,A
 	LD  A,C
 	RRA 
 	RRA 
 	RRA 
 	AND  00011000b
 	OR  H
 	OR  01000000b
 	LD  H,A					;H found
 
 	LD  A,B
 	RRA 
 	RRA 
 	RRA 
 	AND  00011111b
 	LD  L,A
 	LD  A,C
 	RLA 
 	RLA 
 	AND  11100000b
 	OR  L
 	LD  L,A					;L found
 	
 	LD A,B
 	AND 00000111b			;shift found
  	RET 
  	
 ;--------------------------	
OneCharLineDown
;HL: address of screen character line
;go one character line lower
	LD A,L
	ADD A,32
	LD L,A	;go to next line
	RET NC	;if no next screen segment,just return
		
	;go to next screen segment if necessary
	LD A,H
	ADD A,8
	LD H,A
	RET
	
 ;--------------------------	
OneCharLineUp
;HL: address of screen character line
;go one character line upper
	LD A,L
	SUB 32
	LD L,A	;go to next line
	RET NC	;if no next screen segment,just return
		
	;go to next screen segment if necessary
	LD A,H
	SUB 8
	LD H,A
	RET

 ;--------------------
ClearScreen
	XOR A
	OUT (254),A 
						;basic system variable	
	LD (23624),A 		;8-navy,16-red ...
	
	XOR A
	LD HL,16384
	LD (HL),A
	LD DE, 16385
	LD BC, 6144
	LDIR			
	
	LD A,71
	LD (HL),A
	LD BC,767
	LDIR
	RET
	
;---------
ClearScreenBlue
	LD A,69
	LD HL,22528
	LD (HL),A
	LD DE,22529
	LD BC,767
	LDIR
	RET	
	
;--------------------
DrawTile
;HL -pos, DE-tile addr		
	LD A,(DE)
	LD B,A 							;x width
	INC DE
	LD A,(DE)
	LD C,A 							;y-height	
	INC DE			

			
; store screen tile address - HL and its size-BC for further use by attributes routine
	PUSH HL
	PUSH BC			
						
		
;this loop draw "C" lines of chars
DrawTile1	PUSH HL
			PUSH BC
		
;this loop draws one line of chars
DrawTile2					
			LD C,H				;store H
			LD A, (DE)
			CP 16	  			;16-empty char code, when it is detected program jumps to another char.It saves data space	
			JR NZ, DrawTile3	;when first byte is not 16 should draw A char
			
			INC DE 				; when an empty char happens,; so skip it and go to next char 
			JR DrawTile4 
							
					

DrawTile3			
				LD A, (DE)		;draw one char
					LD (HL),A 	;repeated 8 times, quicker than A loop
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC DE
						
DrawTile4		
				LD H,C			;restore H
				INC L			;go one char right  
				DJNZ DrawTile2 
				
			POP BC
			POP HL				
			LD A,L
			ADD A,32			;go to another line of chars 
			LD L,A
			JR NC,DrawTile5		;not carry=no going to next segment, so no H changes needed
				
			LD A,H				;change H only if going to next screen segment
			ADD A,8
			LD H,A
					
DrawTile5			
			DEC C				;decrease number of char lines left to do
			JR NZ, DrawTile1
		
;drawing tile is finished
;now restore size of the tile IN BC, its corner IN HL
; DE is already increased to next byte where attributes data begin
						
		POP BC
		POP HL		
						
;ATTRIBUTES
;BC should contain sprite size (B-width C-height)
;DE should contain start of attributes data		
;HL should contain address of firt byte of attr		
	
;Transform point address to attribute address
		
TileAttrs		
		LD A,80
		SRL H
		SRL H
		SRL H
		ADD A,H
		LD H,A
			
;draw C lines of attributes
TileAttr1
	 	PUSH BC
		PUSH HL
		
;draw one line of attributes
TileAttr2
		LD A,(DE)
		OR A				;check if attribute is set to 0
		JR Z, TileAttr3		;if so dont draw attributes AND leave the square as it is
		LD (HL),A
TileAttr3
		INC HL				;go to next square IN the row
		INC DE
		DJNZ TileAttr2
	
		POP HL
		PUSH DE 			;go to next row of attributes
		LD DE,32
		ADD HL,DE
		POP DE
		POP BC
		DEC C				;decrease number of lines left to do 
		JR NZ, TileAttr1
		RET

;--------------------------------------	
GetPlayerInput
	LD A,(KeysOrJoystick)
	OR A						;keyboard or joystick
	JR Z,KeyboardControl
	
JoystickControl
	LD BC,31
	IN A,(C)
	RET
	
KeyboardControl
;the aim is to set bits of A register as it would be joystick input
	LD D,0
	LD BC,57342; O,P	
	IN A,(C)
	CPL
	AND 00000011B
	LD D,A
	LD BC,64510;Q
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	RLA
	OR D		
	LD D,A		
	LD BC,65022;A
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	OR D
	LD D,A
	LD BC,32766
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	RLA
	RLA
	OR D ;ready
	RET

KeysOrJoystick .BYTE 0

;------------------
CheckQuit
	LD BC,65278			;row with X
	IN A,(C)
	BIT 2,A
	RET NZ				;X not pressed, not quiting
	
;quiting to main menu	
	CALL FadeOut
	JP StartMenu

;------------------
Sound1
	DI
	LD B,50
	LD DE,5
	LD HL,100
Sound1_2	
	PUSH BC
	PUSH DE
	PUSH HL
	
	CALL 949
	
	POP HL
	POP DE
	POP BC
	INC HL
	DJNZ Sound1_2
	EI
	RET
	
;---------
Sound2
	LD BC,64004
	LD HL,512
	LD DE ,2
Sound2_2
	PUSH HL
	PUSH DE
	PUSH BC
	CALL 949
	POP BC
	POP DE
	POP HL
	LD A,L
	SUB C
	LD L,A
	DJNZ Sound2_2
	RET
	
;----------
Sound3	
	DI
	LD H,3
Sound3_0
	SRL A
	SRL A
	SRL A
	LD L,A
	LD E,0
	LD D,32
Sound3_1
	PUSH DE
Sound3_2	
	LD B,E
Sound3_3
	DJNZ Sound3_3
	
	LD A,(BC)
	AND 248
	OR L
	AND 11110000b
	OUT (154),A
	INC C
	DEC D
	JR NZ,Sound3_2
	
	POP DE
	INC E
	DEC D
	JR NZ,Sound3_1
	
	DEC H
	JR NZ,Sound3_0
	EI
	RET

;---------------------
DrawBorder
;draw an empty box with a border (for text or picture to be placed later)
	;HL: position of left upper box corner
	;BC: size,B=width,C=height
	
	CALL DrawBorderA
	CALL DrawBorderB
	CALL DrawBorderC
	RET
	
;-------
DrawBorderA
;draw top edge	
	PUSH BC
	PUSH HL
	LD DE,BorderTL					;left top corner
	CALL DrawTile
	POP HL
	POP BC
	
	INC L							;horizontal line width= border width - 2 (for corners)
	DEC B
	DEC B
;top line
	PUSH BC							;store size
	LD DE,BorderT	
DrawBorderA2
	PUSH HL
	PUSH DE
	PUSH BC
	CALL DrawTile
	POP BC
	POP DE
	POP HL
	INC L
	DJNZ DrawBorderA2
	
	LD DE,BorderTR
	PUSH HL
	CALL DrawTile					;right top corner
	POP HL
	POP BC							;restore size
	RET
	
;-------										
;draw left and right edge
DrawBorderB
	PUSH BC							;store size
	DEC C
	DEC C
	
DrawBorderB2
	CALL OneCharLineDown	

	LD DE,BorderR
	PUSH HL
	PUSH BC							;draw right edge
	CALL DrawTile	
	POP BC
	POP HL
	
	PUSH HL							;store right edge
	
	LD A,L							;go to left edge
	SUB B
	DEC A
	LD L,A							
	
	LD DE,BorderL
	PUSH BC							;draw left edge
	CALL DrawTile	
	POP BC
	
	POP HL							;restore right edge
	
	DEC C							;do it C times
	JR NZ,DrawBorderB2

	POP BC							;restore size
	RET
	
;-------										
;draw bottom edge
DrawBorderC
	CALL OneCharLineDown
	
	LD DE,BorderBR
	PUSH HL
	PUSH BC							;bottom right corner
	CALL DrawTile	
	POP BC
	POP HL

	DEC L							
;bottom line
	LD DE,BorderB	
DrawBorderC2
	PUSH HL
	PUSH DE
	PUSH BC
	CALL DrawTile
	POP BC
	POP DE
	POP HL
	DEC L
	DJNZ DrawBorderC2
	
	LD DE,BorderBL					;bottom left corner
	CALL DrawTile	
	RET

BorderTL
	.BYTE 1,1
	.BYTE	  0, 30, 46, 86,110, 94,124,  0
	.BYTE 70
BorderT
	.BYTE 1,1
	.BYTE	  0,255,  0,255, 85,170,  0,  0
	.BYTE 65
BorderTR
	.BYTE 1,1
	.BYTE	  0,120,116,122,118,122, 62,  0
	.BYTE 70
BorderL
	.BYTE 1,1
	.BYTE	 88, 84, 88, 84, 88, 84, 88, 84
	.BYTE 65
BorderR
	.BYTE 1,1
	.BYTE	 26, 42, 26, 42, 26, 42, 26, 42
	.BYTE 65
BorderBL
	.BYTE 1,1
	.BYTE	  0,124,122,118,122, 62, 30,  0
	.BYTE 70
BorderB
	.BYTE 1,1
	.BYTE	  0,  0,170, 85,255,  0,255,  0
	.BYTE 65
BorderBR
	.BYTE 1,1
	.BYTE	  0, 62,126,118,106, 84,120,  0
	.BYTE 70
	
;----------	
DisplayMoves_Puzzle1	
	HALT
	LD DE,MovesPuzzle1
	LD HL,20480+121
	LD A,(DE)
	CALL DisplayMoves
	RET
	
;----------	
DisplayMoves_Puzzle2	
	HALT
	LD DE,MovesPuzzle2
	LD HL,20480+142
	LD A,(DE)
	CALL DisplayMoves
	RET
	
;----------	
DisplayMoves_Puzzle3	
	HALT
	LD DE,MovesPuzzle3
	LD HL,20480+185
	LD A,(DE)
	CALL DisplayMoves
	RET
;----------------------
DisplayMoves
;DE - adress of cell with number of moves
;HL - position
	
;clear some old grahpics before drawing
	PUSH HL
	INC L
	INC L
	XOR A
	LD B,8
DisplayMoves1
	LD (HL),A
	INC H
	DJNZ DisplayMoves1
	POP HL
				

	LD A,(DE)
	LD B,0		;number of tens

DisplayMoves2	
	INC B
	SUB 10
	JR NC,DisplayMoves2	
	
	DEC B		;there was one increase too much
	ADD A,10
	LD C,A		;number of ones
	
	LD A,B
	PUSH BC
	PUSH HL
	CALL DisplayNumber
	POP HL
	POP BC
	
	LD A,C
	PUSH HL
	INC HL
	CALL DisplayNumber
	POP HL
	
;shift number 5 pixels right to make it centered
	LD B,8
DisplayMoves3
	REPT 5		;repeat 5 times
	SCF
	CCF			;carry=0	
	RR (HL)
	INC L
	RR (HL)
	INC L
	RR (HL)
	DEC L
	DEC L
	ENDR
	INC H
	DJNZ DisplayMoves3
	RET
	
;------
DisplayNumber
;A-number,HL-position
	
;find graphics	
	PUSH HL
	LD HL,Numbers
	LD DE,8
	OR A
	JR Z,DisplayNumber3 ;don't change graphics for A=0

DisplayNumber2	
	ADD HL,DE
	DEC A
	JR NZ,DisplayNumber2	
		
	
DisplayNumber3
	EX HL,DE	;graphics into DE
	POP HL
	
;draw number
	LD B,8
DisplayNumber4
	LD A,(DE)
	LD (HL),A
	INC DE
	INC H
	DJNZ DisplayNumber4
	RET
	
;--------	
MovesPuzzle1	.BYTE 0	
MovesPuzzle2	.BYTE 0	
MovesPuzzle3	.BYTE 0	


;----------------
AttrBlock
;fill area with attribute
;A- attribute
;HL-area corner
;BC-size
	LD DE,32

AttrBlock2
	PUSH HL
	PUSH BC
AttrBlock3	
	LD (HL),A				;one line
	INC L
	DJNZ AttrBlock3
	POP BC
	POP HL
	
	ADD HL,DE				;go one line down
	DEC C
	JR NZ,AttrBlock2
	RET
	
HLGraphToAttr	
	LD A,80
	SRL H
	SRL H
	SRL H
	ADD A,H
	LD H,A					;transform to attribute address
	RET
	